LOGIN: str = 'https://myapi.ku.th/auth/login'
LOGOUT: str = 'https://myapi.ku.th/auth/logout'

GPAX: str = 'https://myapi.ku.th/stddashboard/gpax'

SCHEDULE: str = 'https://myapi.ku.th/common/getschedule'

ANNOUCE: str = 'https://myapi.ku.th/advisor/getAnnounceStd'

GROUP_COURSE: str = 'https://myapi.ku.th/std-profile/getGroupCourse'
CHECK_GRADES: str = 'https://myapi.ku.th/std-profile/checkGrades'
STUDENT_PERSONAL: str = 'https://myapi.ku.th/std-profile/getStdPersonal'
STUDENT_EDUCATION: str = 'https://myapi.ku.th/std-profile/getStdEducation'
STUDENT_ADDRESS: str = 'https://myapi.ku.th/std-profile/getStdAddress'

SEARCH_ENROLL: str = 'https://myapi.ku.th/enroll/searchEnrollResult'
SEARCH_SUBJECT: str = 'https://myapi.ku.th/enroll/searchSubjectOpenEnr'
SEARCH_SUBJECT_OPEN: str = 'https://myapi.ku.th/enroll/openSubjectForEnroll'
SEARCH_SECTION_DETAIL: str = 'https://myapi.ku.th/enroll/searchSectionDetail'
