APP_KEY = 'txCR5732xYYWDGdd49M3R19o1OVwdRFc'

STUDENT_PARAM_NAMES = ('std_id', 'std_code', 'campus_code', 'faculty_code', 'major_code',
                       'student_status_code', 'student_year')

STUDENT_PARAM_RES_NAMES = ('stdId', 'stdCode', 'campusCode', 'facultyCode', 'majorCode',
                           'studentStatusCode', 'studentYear')

STUDENT_PARAM_DICT = dict(zip(STUDENT_PARAM_NAMES, STUDENT_PARAM_RES_NAMES))